@@ -246,7 +246,7 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 		status = add_vevent (cbex, href, lastmod, icalcomp);
 
 		if (status) {
-			char *object = icalcomponent_as_ical_string (icalcomp);
+			char *object = icalcomponent_as_ical_string_r (icalcomp);
 			e_cal_backend_notify_object_created (backend, object);
 			g_free (object);
 		}
@@ -289,7 +289,7 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 			status = add_vevent (cbex, href, lastmod, new_comp);
 
 			if (status) {
-				char *object = icalcomponent_as_ical_string (new_comp);
+				char *object = icalcomponent_as_ical_string_r (new_comp);
 				e_cal_backend_notify_object_created (backend, object);
 				g_free (object);
 			}
@@ -312,6 +312,7 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 
 static const char *event_properties[] = {
 	E2K_PR_CALENDAR_UID,
+	PR_CAL_RECURRING_ID,
 	E2K_PR_DAV_LAST_MODIFIED,
 	E2K_PR_HTTPMAIL_HAS_ATTACHMENT,
 	PR_READ_RECEIPT_REQUESTED,
@@ -335,7 +336,7 @@ get_changed_events (ECalBackendExchange *cbex)
 	E2kRestriction *rn;
 	E2kResultIter *iter;
 	E2kResult *result;
-	const char *prop, *uid, *modtime, *attach_prop, *receipts;
+	const char *prop, *uid, *modtime, *attach_prop, *receipts, *rid;
 	guint status;
 	E2kContext *ctx;
 	int i, status_tracking = EX_NO_RECEIPTS;
@@ -394,6 +395,7 @@ get_changed_events (ECalBackendExchange *cbex)
 			continue;
 		modtime = e2k_properties_get_prop (result->props,
 						   E2K_PR_DAV_LAST_MODIFIED);
+		rid = e2k_properties_get_prop (result->props, PR_CAL_RECURRING_ID);
 
 		attach_prop = e2k_properties_get_prop (result->props,
 						E2K_PR_HTTPMAIL_HAS_ATTACHMENT);
@@ -414,7 +416,7 @@ get_changed_events (ECalBackendExchange *cbex)
 		}
 
 		e_cal_backend_exchange_cache_lock (cbex);
-		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
+		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href, rid)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
 					     g_strdup (modtime));
@@ -575,6 +577,8 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 	GError *error = NULL;
 	ECalBackendExchangeCalendar *cbexc = E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 
+
+
 	/* Do the generic part */
 	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->open_sync (
 		backend, cal, only_if_exists, username, password);
@@ -589,8 +593,8 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_Success;
 
 	e_folder_exchange_subscribe (E_CAL_BACKEND_EXCHANGE (backend)->folder,
-                                        E2K_CONTEXT_OBJECT_CHANGED, 30,
-                                        notify_changes, backend);
+			E2K_CONTEXT_OBJECT_CHANGED, 30,
+			notify_changes, backend);
 
 	thread = g_thread_create ((GThreadFunc) get_changed_events, E_CAL_BACKEND_EXCHANGE (backend), FALSE, &error);
 	if (!thread) {
@@ -682,7 +686,7 @@ find_attendee_prop (icalcomponent *ical_comp, const char *address)
 		const char *attendee;
 		char *text = NULL;
 
-		attendee = icalproperty_get_value_as_string (prop);
+		attendee = icalproperty_get_value_as_string_r (prop);
 		if (!attendee)
 			continue;
 
@@ -737,7 +741,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	struct icaltimetype current;
 	char *location = NULL, *ru_header = NULL;
 	ECalComponent *comp;
-	char *body, *body_crlf, *msg;
+	char *body_crlf, *msg;
 	char *from, *date;
 	const char *summary;
 	char *attach_body = NULL;
@@ -799,9 +803,18 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	send_options = check_for_send_options (icalcomp, props);
 
 	/*set created and last_modified*/
-	current = icaltime_from_timet (time (NULL), 0);
-	icalcomponent_add_property (icalcomp, icalproperty_new_created (current));
-	icalcomponent_add_property (icalcomp, icalproperty_new_lastmodified (current));
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_CREATED_PROPERTY);
+	if (icalprop)
+		icalproperty_set_created (icalprop, current);
+	else
+		icalcomponent_add_property (icalcomp, icalproperty_new_created (current));
+
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_LASTMODIFIED_PROPERTY);
+	if (icalprop)
+		icalproperty_set_lastmodified (icalprop, current);
+	else
+		icalcomponent_add_property (icalcomp, icalproperty_new_lastmodified (current));
 
 	/* Fetch summary */
 	summary = icalcomponent_get_summary (icalcomp);
@@ -852,9 +865,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	icalcomponent_foreach_tzid (real_icalcomp, add_timezone_cb, cbdata);
 	icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);
 
-	body = icalcomponent_as_ical_string (cbdata->vcal_comp);
-	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);
-	g_free (body);
+	body_crlf = icalcomponent_as_ical_string_r (cbdata->vcal_comp);
 
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (cbex->account)))
@@ -1076,7 +1087,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	ECalComponent *real_ecomp, *cached_ecomp = NULL, *updated_ecomp;
 	const char *comp_uid;
 	char *updated_ecomp_str, *real_comp_str;
-	char *body, *body_crlf, *msg;
+	char *body_crlf, *msg;
 	char *attach_body = NULL;
 	char *attach_body_crlf = NULL;
 	char *boundary = NULL;
@@ -1189,7 +1200,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), updated_ecomp);
 
-	last_modified = icaltime_from_timet (time (NULL), 0);
+	last_modified = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 	e_cal_component_set_last_modified (updated_ecomp, &last_modified);
 
 	if (e_cal_component_has_attachments (updated_ecomp)) {
@@ -1375,14 +1386,12 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_backend_exchange_cache_unlock (cbex);
 
 	if (!cached_ecomp && remove)
-		*new_object = icalcomponent_as_ical_string (icalcomp);
+		*new_object = icalcomponent_as_ical_string_r (icalcomp);
 
 	if (!remove && mod == CALOBJ_MOD_THIS)
 		icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);
 
-	body = icalcomponent_as_ical_string (cbdata->vcal_comp);
-	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);
-	g_free (body);
+	body_crlf = icalcomponent_as_ical_string_r (cbdata->vcal_comp);
 
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (cbex->account)))
@@ -1528,7 +1537,7 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		/*remove a single instance of a recurring event and modify */
 		time_rid = icaltime_from_string (rid);
 		e_cal_util_remove_instances (ecomp->icomp, time_rid, mod);
-		calobj  = (char *) icalcomponent_as_ical_string (ecomp->icomp);
+		calobj  = (char *) icalcomponent_as_ical_string_r (ecomp->icomp);
 
 		e_cal_backend_exchange_cache_unlock (cbex);
 		ebs_status = modify_object_with_href (backend, cal, calobj, mod, &obj, &new_object, NULL, rid);
@@ -1627,7 +1636,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		e_cal_component_set_icalcomponent (comp, subcomp);
 
 		/*create time and last modified*/
-		current = icaltime_from_timet (time (NULL), 0);
+		current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 		e_cal_component_set_created (comp, &current);
 		e_cal_component_set_last_modified (comp, &current);
 
@@ -1665,7 +1674,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 					if (e_cal_util_component_is_instance (subcomp))
 						mod = CALOBJ_MOD_THIS;
 
-					icalobj = (char *) icalcomponent_as_ical_string (subcomp);
+					icalobj = (char *) icalcomponent_as_ical_string_r (subcomp);
 					status = modify_object_with_href (backend, cal, icalobj,
 									  mod,
 									  &old_object, &new_object, NULL, NULL);
@@ -1686,7 +1695,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 			} else if (!check_owner_partstatus_for_declined (backend, subcomp)) {
 				char *returned_uid, *object;
 				d(printf ("object : %s .. not found in the cache\n", uid));
-				icalobj = (char *) icalcomponent_as_ical_string (subcomp);
+				icalobj = (char *) icalcomponent_as_ical_string_r (subcomp);
 				d(printf ("Create a new object : %s\n", icalobj));
 
 				e_cal_backend_exchange_cache_unlock (cbex);
@@ -1711,7 +1720,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 			break;
 
 		case ICAL_METHOD_CANCEL:
-			icalobj = (char *) icalcomponent_as_ical_string (subcomp);
+			icalobj = (char *) icalcomponent_as_ical_string_r (subcomp);
 			if (rid)
 				status = remove_object (backend, cal, uid, rid, CALOBJ_MOD_THIS, &icalobj, &object);
 			else
@@ -2009,7 +2018,7 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendSyncStatus retval = GNOME_Evolution_Calendar_Success;
 	ECalBackendExchangeBookingResult result;
 	ECalComponent *comp = NULL;
-	icalcomponent *top_level = NULL, *icalcomp, *tzcomp;
+	icalcomponent *top_level = NULL, *icalcomp;
 	icalproperty *prop;
 	icalproperty_method method;
 	GError *error = NULL;
@@ -2345,7 +2354,7 @@ get_free_busy (ECalBackendSync *backend, EDataCal *cal,
 		content = (char *) fbdata->children->content;
 		set_freebusy_info (vfb, content, start);
 
-		calobj = icalcomponent_as_ical_string (vfb);
+		calobj = icalcomponent_as_ical_string_r (vfb);
 		*freebusy = g_list_prepend (*freebusy, calobj);
 		icalcomponent_free (vfb);
 	}
